@@ -14,10 +14,12 @@ log = logging.getLogger(__name__)
 
 class MatcherWorker(multiprocessing.Process):
 
+    END_TASK = -1
+
     matcher_class = myers.InlineMyersSequenceMatcher
 
     def __init__(self, tasks, results):
-        super(MatcherWorker, self).__init__()
+        super().__init__()
         self.tasks = tasks
         self.results = results
         self.daemon = True
@@ -25,17 +27,18 @@ class MatcherWorker(multiprocessing.Process):
     def run(self):
         while True:
             task_id, (text1, textn) = self.tasks.get()
+            if task_id == self.END_TASK:
+                break
+
             try:
                 matcher = self.matcher_class(None, text1, textn)
                 self.results.put((task_id, matcher.get_opcodes()))
             except Exception as e:
                 log.error("Exception while running diff: %s", e)
-            finally:
-                self.tasks.task_done()
             time.sleep(0)
 
 
-class CachedSequenceMatcher(object):
+class CachedSequenceMatcher:
     """Simple class for caching diff results, with LRU-based eviction
 
     Results from the SequenceMatcher are cached and timestamped, and
@@ -43,6 +46,8 @@ class CachedSequenceMatcher(object):
     eviction is overly simplistic, but is okay for our usage pattern.
     """
 
+    TASK_GRACE_PERIOD = 1
+
     def __init__(self, scheduler):
         """Create a new caching sequence matcher
 
@@ -51,17 +56,25 @@ class CachedSequenceMatcher(object):
         """
         self.scheduler = scheduler
         self.cache = {}
-        self.tasks = multiprocessing.JoinableQueue()
+        self.tasks = multiprocessing.Queue()
+        self.tasks.cancel_join_thread()
         # Limiting the result queue here has the effect of giving us
         # much better interactivity. Without this limit, the
         # result-checker tends to get starved and all highlights get
         # delayed until we're almost completely finished.
-        self.results = multiprocessing.JoinableQueue(5)
+        self.results = multiprocessing.Queue(5)
+        self.results.cancel_join_thread()
         self.thread = MatcherWorker(self.tasks, self.results)
         self.task_id = 1
         self.queued_matches = {}
         GLib.idle_add(self.thread.start)
 
+    def __del__(self):
+        self.tasks.put((MatcherWorker.END_TASK, ('', '')))
+        self.thread.join(self.TASK_GRACE_PERIOD)
+        if self.thread.exitcode is None:
+            self.thread.terminate()
+
     def match(self, text1, textn, cb):
         texts = (text1, textn)
         try:
